@@ -3,6 +3,8 @@ from __future__ import print_function
 import json
 import time
 import requests
+import base64
+from time import sleep, time
 
 from dltk_ai.dataset_types import Dataset
 
@@ -160,68 +162,185 @@ class DltkAiClient:
 
     # phoenix-vision
 
-    def face_detection_image(self, image_path):
+    #def face_detection_image(self, image_path):
+    #    """
+    #    :param str image_path: The path of the image file.
+    #    :return
+    #        text : A base64 decoded image with face detected.
+    #    """
+
+    #    body = {'file': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
+    #    url = self.base_url + '/vision/face-detection/image'
+    #    headers = {'ApiKey': self.api_key}
+    #    response = requests.post(url=url, files=body, headers=headers)
+    #    return response.content
+
+    #def face_detection_image_core(self, image_path):
+    #    """
+    #    :param str image_path: The path of the image file.
+    #    :return
+    #        text : A base64 decoded image with face detected.
+    #    """
+    #
+    #    body = {'image': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
+    #    url = self.base_url + '/core/vision/face-detection/image'
+    #    headers = {'ApiKey': self.api_key}
+    #    response = requests.post(url=url, files=body, headers=headers)
+    #    return response.content
+
+    #def face_detection_json(self, image_path):
+    #    """
+    #    :param str image_path: The path of the image file.
+    #    :return
+    #        obj : A list of co-ordinates for all faces detected in the image.
+    #    """
+    #    body = {'file': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
+    #    url = self.base_url + '/vision/face-detection/json'
+    #    headers = {'ApiKey': self.api_key}
+    #    response = requests.post(url=url, files=body, headers=headers).json()
+    #    return response
+
+    #def face_detection_json_core(self, image_path):
+    #    """
+    #    :param str image_path: The path of the image file.
+    #    :return
+    #        obj : A list of co-ordinates for all faces detected in the image.
+    #    """
+    #    body = {'image': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
+    #    url = self.base_url + '/core/vision/face-detection/json'
+    #    headers = {'ApiKey': self.api_key}
+    #    response = requests.post(url=url, files=body, headers=headers).json()
+    #    return response
+
+    #def face_detection_compare(self, image_path,sources):
+    #    """
+    #    :param str image_path: The path of the image file.
+    #    :return
+    #        obj : A list of co-ordinates for all objects detected in the image.
+    #    """
+    #    body = {'image': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
+    #    payload = {'sources':sources}
+    #    url =  self.base_url + '/core/vision/face-detection/compare'
+    #    headers = {'ApiKey': self.api_key}
+    #    response = requests.post(url=url, files=body, headers=headers,data=payload)
+    #    return response
+
+    # Note: Computer vision functions
+    def check_cv_job_status(self, task_creation_response):
         """
-        :param str image_path: The path of the image file.
-        :return
-            text : A base64 decoded image with face detected.
+        This function check status of the job
+        Args:
+            task_creation_response: response from task creation function
+
+        Returns:
+
         """
+        headers = {'ApiKey': self.api_key, 'Content-type': 'application/json'} 
+        task_url = f"{self.base_url}/computer_vision/task?task_id="
+        task_status_response = {}
+        # ensure the request was successful
+        if task_creation_response.status_code == 200:
+            task_creation_response = task_creation_response.json()
+            task_status = "PENDING"
+
+            start_time = time()
+            while task_status != "SUCCESS":
+                task_status_response = requests.get(task_url + task_creation_response['job_id'], headers=headers).json()
+                task_status = task_status_response["task_status"]
+                # wait for some time before checking status of the job again
+                sleep(1)
+
+                # check if execution time is more than
+                if time() - start_time > 10:
+                    print("It's taking too long than expected!!",
+                          'you can use this function to check status dltkai.check_cv_job(',
+                          task_creation_response['job_id'], ')')
+                    break
+
+        else:
+            print(f"FAILED due to {task_creation_response.content}")
+        return task_status_response
+
+    def check_cv_job(self, job_id):
+        """
+               This function check status of the job
+               Args:
+                   job_id: job_id from check_cv_job_status will be returned if its taking too long
+
+               Returns:
+                   task_status_response: contains the result requested by the cv methods (only used when cv_job_status
+                    crosses 10 second threshold)
+
+               """
+
+        headers = {'ApiKey': self.api_key, 'Content-type': 'application/json'}
+        task_url = f"{self.base_url}/computer_vision/task?task_id="
+        task_status_response = requests.get(task_url + str(job_id), headers=headers).json()
+        return task_status_response
+
+    def face_analytics(self, image_url=None, features=None, image_path=None, dlib=False, opencv=True,
+                       azure=False, mtcnn=False,
+                       output_types=["json"]):
+        """
+        This function is for face analytics
+        Args:
+            output_types (list): Type of output requested by client: "json", "image"
+            image_url: Image URL
+            image_path: Local Image Path
+            features (list) : Type of features requested by client
+            dlib: if True, uses dlib for face analytics
+            opencv: if True, uses opencv for face analytics
+            azure: if True, returns azure results of face analytics on given image
+            mtcnn: if True, uses mtcnn for face analytics
+
+        Returns:
+        face analytics response dependent on the features requested by client
+        """
+        if features is None:
+            features = ['face_locations']
+        features = [feature.lower() for feature in features]
+        assert image_url is not None or image_path is not None, "Please choose either image_url or image_path"
+        assert any(
+            (azure, mtcnn, dlib, opencv)), "please choose at least 1 processor ['opencv', 'azure', 'mtcnn', 'dlib']"
+        assert "json" in output_types or "image" in output_types, "Please select at least 1 output type ['json','image']"
+        assert "face_locations" in features, "Please select at least one feature ['face_locations']"
+        load = {
+            "image_url": image_url,
+
+            "tasks": {"face_detection": False},
+
+            "configs": {
+                "output_types": output_types,
+
+                "face_detection_config": {
+                    "dlib": dlib,
+                    "opencv": opencv,
+                    "mtcnn": mtcnn,
+                    "azure": azure
+                }
+            }
+        }
+
+        if 'face_locations' in features:
+            load['tasks']['face_detection'] = True
+        if image_url is not None and image_path is None:
+            load["input_method"] = "image_url"
+            load["image_url"] = image_url
+
+        elif image_url is None and image_path is not None:
+            with open(image_path, "rb") as image_file:
+                base64_img = base64.b64encode(image_file.read()).decode('utf-8')
+            load["base64_img"] = base64_img
+            load["input_method"] = "base64_img"
 
-        body = {'file': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
-        url = self.base_url + '/vision/face-detection/image'
         headers = {'ApiKey': self.api_key}
-        response = requests.post(url=url, files=body, headers=headers)
-        return response.content
+        url = self.base_url + '/computer_vision/face_analytics/'
 
-    def face_detection_image_core(self, image_path):
-        """
-        :param str image_path: The path of the image file.
-        :return
-            text : A base64 decoded image with face detected.
-        """
+        task_response = requests.post(url, json=load, headers=headers)
+        response = self.check_cv_job_status(task_response)
 
-        body = {'image': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
-        url = self.base_url + '/core/vision/face-detection/image'
-        headers = {'ApiKey': self.api_key}
-        response = requests.post(url=url, files=body, headers=headers)
-        return response.content
-
-    def face_detection_json(self, image_path):
-        """
-        :param str image_path: The path of the image file.
-        :return
-            obj : A list of co-ordinates for all faces detected in the image.
-        """
-        body = {'file': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
-        url = self.base_url + '/vision/face-detection/json'
-        headers = {'ApiKey': self.api_key}
-        response = requests.post(url=url, files=body, headers=headers).json()
         return response
 
-    def face_detection_json_core(self, image_path):
-        """
-        :param str image_path: The path of the image file.
-        :return
-            obj : A list of co-ordinates for all faces detected in the image.
-        """
-        body = {'image': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
-        url = self.base_url + '/core/vision/face-detection/json'
-        headers = {'ApiKey': self.api_key}
-        response = requests.post(url=url, files=body, headers=headers).json()
-        return response
-
-    def face_detection_compare(self, image_path,sources):
-        """
-        :param str image_path: The path of the image file.
-        :return
-            obj : A list of co-ordinates for all objects detected in the image.
-        """
-        body = {'image': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
-        payload = {'sources':sources}
-        url =  self.base_url + '/core/vision/face-detection/compare'
-        headers = {'ApiKey': self.api_key}
-        response = requests.post(url=url, files=body, headers=headers,data=payload)
-        return response
 
     def eye_detection_image(self, image_path):
         """
@@ -299,66 +418,169 @@ class DltkAiClient:
         return response
 
 
-    def object_detection_image(self, image_path):
-        """
-        :param str image_path: The path of the image file.
-        :return
-            text : A base64 decoded image with objects detected.
-        """
-        body = {'image': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
-        url = self.base_url + '/core/vision/object-detection/image'
-        headers = {'ApiKey': self.api_key}
-        response = requests.post(url=url, files=body, headers=headers)
-        return response.content
+    #def object_detection_image(self, image_path):
+    #    """
+    #    :param str image_path: The path of the image file.
+    #    :return
+    #        text : A base64 decoded image with objects detected.
+    #    """
+    #    body = {'image': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
+    #    url = self.base_url + '/core/vision/object-detection/image'
+    #    headers = {'ApiKey': self.api_key}
+    #    response = requests.post(url=url, files=body, headers=headers)
+    #    return response.content
 
-    def object_detection_json(self, image_path):
+    #def object_detection_json(self, image_path):
+    #    """
+    #    :param str image_path: The path of the image file.
+    #    :return
+    #        obj : A list of co-ordinates for all objects detected in the image.
+    #    """
+    #    body = {'image': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
+    #    url = self.base_url + '/core/vision/object-detection/json'
+    #    headers = {'ApiKey': self.api_key}
+    #    response = requests.post(url=url, files=body, headers=headers).json()
+    #    return response
+
+    #def object_detection_compare(self, image_path,sources):
+    #    """
+    #    :param str image_path: The path of the image file.
+    #    :return
+    #        obj : A list of co-ordinates for all objects detected in the image.
+    #    """
+    #    body = {'image': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
+    #    payload = {'sources':sources}
+    #    url = self.base_url + '/core/vision/object-detection/image/compare'
+    #    headers = {'ApiKey': self.api_key}
+    #    response = requests.post(url=url, files=body, headers=headers,data=payload)
+    #    return response
+
+    #def image_classification(self, image_path):
+    #    """
+    #    :param str image_path: The path of the image file.
+    #    :return
+    #        obj : A list of all classes detected in the image.
+    #    """
+    #    body = {'image': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
+    #    url = self.base_url + '/core/vision/image-classification'
+    #    headers = {'ApiKey': self.api_key}
+    #    response = requests.post(url=url, files=body, headers=headers).json()
+    #    return response
+
+    #def image_classification_compare(self, image_path,sources):
+    #    """
+    #    :param str image_path: The path of the image file.
+    #    :return
+    #        obj : A list of co-ordinates for all objects detected in the image.
+    #    """
+    #    body = {'image': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
+    #    payload = {'sources':sources}
+    #    url = self.base_url + '/core/vision/image-classification/compare'
+    #    headers = {'ApiKey': self.api_key}
+    #    response = requests.post(url=url, files=body, headers=headers,data=payload)
+    #    return response
+
+    def object_detection(self, image_url=None, image_path=None, tensorflow=True, azure=False, output_types=["json"]):
         """
-        :param str image_path: The path of the image file.
-        :return
-            obj : A list of co-ordinates for all objects detected in the image.
+        This function is for object detection
+        Args:
+            output_types (list): Type of output requested by client: "json", "image"
+            image_url: Image URL
+            image_path: Local Image Path
+            tensorflow: if True, uses tensorflow for object detection
+            azure: if True, returns azure results of object detection on given image
+
+        Returns:
+        Object_detection
         """
-        body = {'image': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
-        url = self.base_url + '/core/vision/object-detection/json'
+
+        assert image_url is not None or image_path is not None, "Please choose either image_url or image_path"
+        assert tensorflow is True or azure is True, "please choose at least 1 supported processor ['tensorflow', 'azure']"
+        assert "json" in output_types or "image" in output_types, "Please select at least 1 output type"
+
+        load = {
+            "tasks": {"object_detection": True},
+            "configs": {
+                "output_types": output_types,
+                "object_detection_config": {
+                    "tensorflow": tensorflow,
+                    "azure": azure
+                }
+
+            }
+        }
+
+        if image_url is not None and image_path is None:
+            load["input_method"] = "image_url"
+            load["image_url"] = image_url
+
+        elif image_url is None and image_path is not None:
+            with open(image_path, "rb") as image_file:
+                base64_img = base64.b64encode(image_file.read()).decode('utf-8')
+                load["base64_img"] = base64_img
+                load["input_method"] = "base64_img"
+
         headers = {'ApiKey': self.api_key}
-        response = requests.post(url=url, files=body, headers=headers).json()
+        url = self.base_url + '/computer_vision/object_detection/'
+
+        task_response = requests.post(url, json=load, headers=headers)
+        response = self.check_cv_job_status(task_response)
+
         return response
 
-    def object_detection_compare(self, image_path,sources):
-        """
-        :param str image_path: The path of the image file.
-        :return
-            obj : A list of co-ordinates for all objects detected in the image.
-        """
-        body = {'image': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
-        payload = {'sources':sources}
-        url = self.base_url + '/core/vision/object-detection/image/compare'
-        headers = {'ApiKey': self.api_key}
-        response = requests.post(url=url, files=body, headers=headers,data=payload)
-        return response
 
-    def image_classification(self, image_path):
+    def image_classification(self, image_url=None, image_path=None, top_n=3, tensorflow=True, azure=False, ibm=False,
+                             output_types=["json"]):
         """
-        :param str image_path: The path of the image file.
-        :return
-            obj : A list of all classes detected in the image.
-        """
-        body = {'image': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
-        url = self.base_url + '/core/vision/image-classification'
-        headers = {'ApiKey': self.api_key}
-        response = requests.post(url=url, files=body, headers=headers).json()
-        return response
+        This function is for image classification
+        Args:
 
-    def image_classification_compare(self, image_path,sources):
+            top_n: get top n predictions
+            output_types (list): Type of output requested by client: "json", "image"
+            image_url: Image URL
+            image_path: Local Image Path
+            tensorflow: if True, uses tensorflow for object detection
+            azure: if True, returns azure results of object detection on given image
+            ibm: if True, returns ibm results of object detection on given image
+
+        Returns:
+        Image classification response
         """
-        :param str image_path: The path of the image file.
-        :return
-            obj : A list of co-ordinates for all objects detected in the image.
-        """
-        body = {'image': (image_path, open(image_path, 'rb'), 'multipart/form-data')}
-        payload = {'sources':sources}
-        url = self.base_url + '/core/vision/image-classification/compare'
+
+        assert image_url is not None or image_path is not None, "Please choose either image_url or image_path"
+        assert tensorflow is True or azure is True or ibm is True, "please choose at least 1 supported processor ['tensorflow', 'azure','ibm']"
+        assert "json" in output_types or "image" in output_types, "Please select at least 1 output type"
+
+        load = {
+            "tasks": {"image_classification": True},
+
+            "configs": {
+                "output_types": ["json"],
+                "img_classification_config": {
+                    "top_n": top_n,
+                    "tensorflow": tensorflow,
+                    "ibm": ibm,
+                    "azure": azure
+                }
+            }
+        }
+
+        if image_url is not None and image_path is None:
+            load["input_method"] = "image_url"
+            load["image_url"] = image_url
+
+        elif image_url is None and image_path is not None:
+            with open(image_path, "rb") as image_file:
+                base64_img = base64.b64encode(image_file.read()).decode('utf-8')
+            load["base64_img"] = base64_img
+            load["input_method"] = "base64_img"
+
         headers = {'ApiKey': self.api_key}
-        response = requests.post(url=url, files=body, headers=headers,data=payload)
+        url = self.base_url + '/computer_vision/image_classification'
+
+        task_response = requests.post(url, json=load, headers=headers)
+        response = self.check_cv_job_status(task_response)
+
         return response
 
     def custom_image_clf_train(self, name, zip_folder_url, images_column, labels_column, train_percentage, epochs, steps_per_epoch):
